@@ -113,7 +113,8 @@ they are not translated.
 my @comments = ();
 
 my %debug=('split_attributelist' => 0,
-           'join_attributelist'  => 0
+           'join_attributelist'  => 0,
+           'parse'               => 0,
            );
 
 sub initialize {
@@ -257,12 +258,12 @@ BEGIN {
     my $UnicodeGCString_available = 0;
     $UnicodeGCString_available = 1 if (eval { require Unicode::GCString });
     eval {
-        sub columns($$$) {
+        sub chars($$$) {
             my $text = shift;
             my $encoder = shift;
             $text = $encoder->decode($text) if (defined($encoder) && $encoder->name ne "ascii");
             if ($UnicodeGCString_available) {
-                return Unicode::GCString->new($text)->columns();
+                return Unicode::GCString->new($text)->chars();
             } else {
                 $text =~ s/\n$//s;
                 return length($text) if !(defined($encoder) && $encoder->name ne "ascii");
@@ -291,6 +292,8 @@ sub parse {
         }
 
         chomp($line);
+        print STDERR "Seen $ref $line\n"
+	    if ($debug{parse});
         $self->{ref}="$ref";
         if ((defined $self->{verbatim}) and ($self->{verbatim} == 3)) {
             # Untranslated blocks
@@ -313,12 +316,16 @@ sub parse {
             # List Item Continuation or List Block
             do_paragraph($self,$paragraph,$wrapped_mode);
             $paragraph="";
+	    $wrapped_mode = 1 unless defined($self->{verbatim});
             $self->pushline($line."\n");
+            # TODO: add support for Open blocks
         } elsif ((not defined($self->{verbatim})) and
                  ($line =~ m/^(={2,}|-{2,}|~{2,}|\^{2,}|\+{2,})$/) and
                  (defined($paragraph) )and
                  ($paragraph =~ m/^[^\n]*\n$/s) and
-                 (columns($paragraph, $self->{TT}{po_in}{encoder}, $ref) == (length($line)))) {
+                 # subtract one because chars includes the newline on the paragraph
+                 ((chars($paragraph, $self->{TT}{po_in}{encoder}, $ref) - 1) == (length($line)))) {
+
             # Found title
             $wrapped_mode = 0;
             my $level = $line;
@@ -333,7 +340,7 @@ sub parse {
             $paragraph="";
             @comments=();
             $wrapped_mode = 1;
-            $self->pushline(($level x (columns($t, $self->{TT}{po_in}{encoder}, $ref)))."\n");
+            $self->pushline(($level x (chars($t, $self->{TT}{po_in}{encoder}, $ref)))."\n");
         } elsif ($line =~ m/^(={1,5})( +)(.*?)( +\1)?$/) {
             my $titlelevel1 = $1;
             my $titlespaces = $2;
@@ -351,7 +358,7 @@ sub parse {
             $self->pushline($titlelevel1.$titlespaces.$t.$titlelevel2."\n");
             @comments=();
             $wrapped_mode = 1;
-        } elsif ($line =~ m/^(\/{4,}|\+{4,}|-{4,}|\.{4,}|\*{4,}|_{4,}|={4,}|~{4,}|\|={4,})$/) {
+        } elsif ($line =~ m/^(\/{4,}|\+{4,}|-{4,}|\.{4,}|\*{4,}|_{4,}|={4,}|~{4,}|\|={3,})$/) {
             # Found one delimited block
             my $t = $line;
             $t =~ s/^(.).*$/$1/;
@@ -364,8 +371,12 @@ sub parse {
                     and ($self->{type} eq $type)) {
                     undef $self->{type};
                     undef $self->{verbatim};
+		    undef $self->{bullet};
+		    undef $self->{indent};
                     $wrapped_mode = 1;
+		    print STDERR "Closing $t block\n" if  $debug{parse};
                 } else {
+		    print STDERR "Begining $t block\n" if $debug{parse};
                     if ($t eq "\/") {
                         # CommentBlock, should not be treated
                         $self->{verbatim} = 2;
@@ -390,6 +401,7 @@ sub parse {
                             and ($self->{type} eq "verse")) {
                             $wrapped_mode = 0;
                             $self->{verbatim} = 1;
+			    print STDERR "QuoteBlock verse\n" if $debug{parse};
                         } else {
                             $wrapped_mode = 1;
                         }
@@ -456,8 +468,11 @@ sub parse {
             @comments=();
             $wrapped_mode = 1;
             if ($line =~ m/^\[(['"]?)(verse|quote)\1,/) {
-                $wrapped_mode = 0 if $2 eq 'verse';
                 $self->{type} = $2;
+		if ($self->{type} eq 'verse') {
+		    $wrapped_mode = 0;
+		}
+		print STDERR "Starting verse\n" if $debug{parse};
             }
             undef $self->{bullet};
             undef $self->{indent};
@@ -527,7 +542,7 @@ sub parse {
             }
             @comments=();
         } elsif (not defined $self->{verbatim} and
-                 ($line =~ m/^([\w\d][\w\d-]*)(::)(\S+)\[(.*)\]$/)) {
+                 ($line =~ m/^([\w\d][\w\d-]*)(::)(\S*)\[(.*)\]$/)) {
             my $macroname = $1;
             my $macrotype = $2;
             my $macrotarget = $3;
@@ -564,6 +579,7 @@ sub parse {
             my $indent = $1||"";
             my $bullet = $2;
             my $text = $3;
+	    print STDERR "Item (bullet: '$bullet')\n" if ($debug{parse});
             do_paragraph($self,$paragraph,$wrapped_mode);
             $paragraph = $text."\n";
             $self->{indent} = $indent;
@@ -576,28 +592,45 @@ sub parse {
             $paragraph = $text."\n";
             $self->{indent} = "";
             $self->{bullet} = $bullet;
+        } elsif ($line =~ /^\s*$/) {
+            # Break paragraphs on empty lines or lines containing only spaces
+	    print STDERR "Empty new line. Wrap: ".(defined($self->{verbatim})?"yes. ":"no. ")."\n"
+		if $debug{parse};
+            do_paragraph($self,$paragraph,$wrapped_mode);
+            $paragraph="";
+	    $wrapped_mode = 1 unless defined($self->{verbatim});
+            $self->pushline($line."\n");
         } elsif (not defined $self->{verbatim} and
                  (defined $self->{bullet} and $line =~ m/^(\s+)(.*)$/)) {
             my $indent = $1;
             my $text = $2;
+	    print STDERR "bullet (".($self->{bullet}).") starting with ".length($indent)." spaces\n"
+		if $debug{'parse'};
+	    if ($paragraph eq "" && length($self->{bullet}) && length($indent)) {
+		# starting a paragraph with a bullet (not an enum or so), and indented.
+		# Thus a literal paragraph in a list.
+		$wrapped_mode = 0;
+	    }
             if (not defined $self->{indent}) {
+		# No indent level before => Starting a paragraph?
                 $paragraph .= $text."\n";
                 $self->{indent} = $indent;
+		print STDERR "Starting a paragraph\n" if ($debug{parse});
             } elsif (length($paragraph) and (length($self->{bullet}) + length($self->{indent}) == length($indent))) {
+		# same indent level as before: append
                 $paragraph .= $text."\n";
             } else {
+		# not the same indent level: start a new translated paragraph
+		print STDERR "New paragraph (indent: '".($self->{indent})."')\n" if ($debug{parse});
                 do_paragraph($self,$paragraph,$wrapped_mode);
+		if (length($self->{indent})>0 && length($self->{indent}) < length($indent)) {
+		    # increase indentation: the new block must not be wrapped
+		    $wrapped_mode = 0;
+		}
                 $paragraph = $text."\n";
                 $self->{indent} = $indent;
                 $self->{bullet} = "";
             }
-        } elsif ($line =~ /^\s*$/) {
-            # Break paragraphs on lines containing only spaces
-            do_paragraph($self,$paragraph,$wrapped_mode);
-            $paragraph="";
-            $wrapped_mode = 1 unless defined($self->{verbatim});
-            $self->pushline($line."\n");
-            undef $self->{controlkey};
         } elsif ($line =~ /^-- $/) {
             # Break paragraphs on email signature hint
             do_paragraph($self,$paragraph,$wrapped_mode);
@@ -613,14 +646,28 @@ sub parse {
             $paragraph="";
             $wrapped_mode = 1;
         } else {
+	    # A stupid paragraph of text
+	    print STDERR "Regular line. ".
+		"Bullet: '".(defined($self->{bullet})?$self->{bullet}:'none')."'; ".
+		"Indent: '".(defined($self->{indent})?$self->{indent}:'none')."'\n"
+		if ($debug{parse});
+
             if ($line =~ /^\s/) {
                 # A line starting by a space indicates a non-wrap
                 # paragraph
                 $wrapped_mode = 0;
             }
-            undef $self->{bullet};
-            undef $self->{indent};
-    # TODO: comments
+
+	    if ($paragraph ne "" && $self->{bullet} && length($self->{indent}||"")==0) {
+		# Second line of an item block is not indented. It looks like a broken indentation
+		# I'd prefer not to accept it, but the formaters do. Damn specification
+		print STDERR "$ref: It seems that you are adding unindented content to an item.\n".
+		    "The \"standard\" allows this, but you may still want to fix your document.\n";
+	    } else {
+		undef $self->{bullet};
+		undef $self->{indent};
+	    }
+            # TODO: comments
             $paragraph .= $line."\n";
         }
         # paragraphs starting by a bullet, or numbered
@@ -744,7 +791,9 @@ sub parse_macro {
                          "wrap" => 0);
         return $t;
     }
-    my @attributes = $self->split_attributelist($macroparam);
+    my @attributes = ();
+    @attributes = $self->split_attributelist($macroparam) unless $macroparam eq "";
+
     unshift @attributes, $macroname;
     my @translated_attributes = $self->join_attributelist("macro", @attributes);
     shift @translated_attributes;
@@ -771,13 +820,12 @@ sub split_attributelist {
        | [^\W\d][-\w]*=\S+                  # invalid, but accept it anyway
        | "(?:[^"\\]++|\\.)*+"               # quoted attribute
        |  (?:[^,\\]++|\\.)++                # unquoted attribute
+       | ^$                                 # Empty attribute list allowed
+
          )(?:,\s*+)?/gx) {
         print STDERR "  -> $1\n" if $debug{split_attributelist};
         push @attributes, $1;
     }
-    die wrap_mod("po4a::asciidoc",
-                 dgettext("po4a", "Unable to parse attribute list: [%s]"), $text)
-            unless scalar(@attributes);
     return @attributes;
 }
 
@@ -785,7 +833,12 @@ sub join_attributelist {
     my ($self, $type) = (shift, shift);
     my @attributes = @_;
     my $command = shift(@attributes);
-    my $position = 1;
+    my $position;
+    if ($type eq 'macro') {
+        $position = 0; # macroname is passed through the first attribute
+    } else {
+        $position = 1;
+    }
     my @text = ($command);
     if ($command =~ m/=/) {
         my $attr = $command;
@@ -874,6 +927,7 @@ Tested successfully on simple AsciiDoc files.
 
  Copyright 2005-2008 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
  Copyright 2012 by Denis BARBIER <barbier@linuxfr.org>.
+ Copyright 2017 by Martin Quinson <mquinson#debian.org>.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
