@@ -311,7 +311,7 @@ or using an alias:
 
  .IR_untranslated \-q ", " \-\-quiet
 
-(this will require the B<-o untranslated=als,IR_untranslated> option)
+This will require the B<-o untranslated=als,IR_untranslated> option.
 
 =head2 Conclusion
 
@@ -324,7 +324,7 @@ it's valid in nroff, but would complicate too much the parser to be
 handled.
 
 Of course, another possibility is to use another format, more translator
-friendly (like POD using po4a::pod, or one of the XML familly like SGML),
+friendly (like POD using po4a::pod, or one of the XML family like SGML),
 but thanks to po4a::man it isn't needed anymore. That being said, if the
 source format of your documentation is POD, or XML, it may be clever to
 translate the source format and not this generated one. In most cases,
@@ -338,7 +338,7 @@ In most cases, po4a::man will detect the problem and refuse to process the
 page, issuing an adapted message. In some rare cases, the program will
 complete without warning, but the output will be wrong. Such cases are
 called "bugs" ;) If you encounter such case, be sure to report this, along
-with a fix when possible...
+with a fix when possible…
 
 =head1 STATUS OF THIS MODULE
 
@@ -642,7 +642,7 @@ NEW_LINE:
     #     of the module (e.g. when searching for a font : \f, whe don't
     #     want to match \\f)
     $line =~ s/\\\\/\\e/g;
-    #   * \. is just a dot (this can even be use to introduce a macro)
+    #   * \. is just a dot (this can even be used to introduce a macro)
     $line =~ s/\\\././g;
 
     chomp $line;
@@ -686,8 +686,8 @@ NEW_LINE:
 
         if ($line =~ m/^[.']*$/) {
             if ($c !~ m/^\s+$/) {
-                # This commentted line may be comment for the next paragraph
-                push @next_comments, [substr($line,0,1),$c];
+                # This commented line may be comment for the next paragraph
+                push @next_comments, [$line ? substr($line,0,1) : '.', $c];
             }
             if ($line =~ m/^[.']+$/) {
                 # those lines are ignored
@@ -1968,10 +1968,41 @@ $macro{'IP'}=sub {
 #         corresponding UE  command.
 #         When generating HTML this should translate into the HTML command
 #         <A  NAME="u" id="u">&nbsp;</A>
-$inline{'UR'}=1;
-$inline{'UE'}=1;
+#
+# E-Mail address Macros
+#  .MT m  Begins a mailto link to the adress m; it will end with
+#         the corresponding ME command. When generating HTML this should
+#         translate into the HTML command <A HREF="mailto:m">.
+#  .ME    Ends the corresponding MT command; when generating HTML this
+#         should translate into </A>.
+$inline{'UR'}=$inline{'MT'}=1;
+$inline{'UE'}=$inline{'ME'}=1;
 $macro{'UN'}=\&translate_joined;
 
+# Macros to describe command synopses
+#
+# These macros are a convenience for authors.  They also assist
+# automated translation tools and help browsers in recognizing command
+# synopses and treating them differently from running text.
+#
+# .OP key value
+#       Describe an optional command argument.  The arguments of this
+#       macro are set surrounded by option braces in the default Roman
+#       font; the first argument is printed with a bold face, while
+#       the second argument is typeset as italic.
+#
+# .SY command
+#       Begin synopsis.  Takes a single argument, the name of a
+#       command.  Text following, until closed by .YS, is set with a
+#       hanging indentation with the width of command plus a space.
+#       This produces the traditional look of a Unix command synopsis.
+#
+# .YS    This macro restores normal indentation at the end of a command
+#       synopsis.
+$macro{'OP'}=\&translate_each;
+$macro{'SY'}=\&translate_joined;
+$macro{'YS'}=\&noarg;
+
 # Miscellaneous Macros
 #  .DT      Reset tabs to default tab values (every 0.5 inches); does not
 #           cause a break.
@@ -2190,9 +2221,10 @@ $macro{'ti'}=\&untranslated;
 ###
 $macro{'TS'}=sub {
     my $self=shift;
-    my ($in_headers,$buffer)=(1,"");
+    my ($in_headers,$tab,$buffer)=(1,"\t","");
     my ($in_textblock,$preline,$postline)=(0,"","");
     my ($line,$ref)=$self->shiftline();
+    my @options;
 
     # Push table start
     $self->pushmacro(@_);
@@ -2203,7 +2235,11 @@ $macro{'TS'}=sub {
             return;
         }
         if ($in_headers) {
-            if ($line =~ /\.$/) {
+            if ($line =~ /;$/) { # global options line
+                if ($line =~ /\btab\s*\((.)\)/) {
+                    $tab = $1;
+                }
+            } elsif ($line =~ /\.$/) {
                 $in_headers = 0;
             }
             $self->pushline($self->r($line));
@@ -2211,10 +2247,16 @@ $macro{'TS'}=sub {
             $in_textblock = 0;
             $preline = $&; # save the `T}' marker to be output later
             $line = $';    # save the remaing part of the line
+            # Drop any EOL from entry to be translated and save it for
+            # output below.
+            if (chomp $buffer) {
+                $postline .= "\n";
+            }
             $self->pushline($self->translate($buffer,
                                              $ref,
-                                            'tbl table'));
-            $buffer = "";
+                                            'tbl table')
+                            .$postline);
+            $buffer = $postline = "";
             next; # continue processing with the remaining part of the line
         } elsif ($in_textblock && $line =~ /^[.']/) {
             # TODO: properly handle macros inside text blocks, currently we mark them
@@ -2230,26 +2272,31 @@ $macro{'TS'}=sub {
             # Lines are continued on \ at the end of line
             $buffer .= $line;
         } else {
-            if ($line =~ s/\s*T{\s*$//) { # start of text block
+            if ($line =~ s/\s*T\{\s*$//) { # start of text block
               $in_textblock = 1;
               $postline = $&; # save the `T{' to be outputed below
-            } elsif ($buffer eq "" && $line ne ""){ # single line data
-              chomp $line; # drop eol char from the entry to be translated
-              $postline = "\n"; # and save the eol for output below
             }
 
             $buffer .= $line;
-            # Arguments to translate are separated by \t
+            # Drop any EOL from entry to be translated and save it for
+            # output below.
+            if (chomp $buffer) {
+                $postline .= "\n";
+            }
+            # Arguments to translate are separated by the table's tab
+            # character (\t by default).  We must be careful to preserve
+            # empty trailing fields, since in particular a text block is
+            # likely to show up as an empty trailing field here.
             $self->pushline($preline
-                            .join("\t",
+                            .join($tab,
                                  map { $self->translate($_,
                                                         $ref,
                                                         'tbl table')
-                                     } split (/\t/,$buffer))
+                                     } split (/\Q$tab/,$buffer,-1))
                            .$postline);
 
             $buffer = $preline = $postline = "";
- 
+
         }
         ($line,$ref)=$self->shiftline();
     }
