@@ -11,7 +11,7 @@ use warnings;
 
 use subs qw(makespace);
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION="0.48";
+$VERSION="0.54";
 @ISA = qw(DynaLoader);
 @EXPORT = qw(new process translate
              read write readpo writepo
@@ -501,6 +501,11 @@ of use:
     ($percent,$hit,$queries) = $document->stats();
     print "We found translations for $percent\%  ($hit from $queries) of strings.\n";
 
+=item is_po_uptodate()
+
+Returns ($uptodate, $diagnostic) where $uptodate is whether the input po and the output po match (if not, it means that the input po should be updated)
+and $diagnostic is a string explaining why the po file is not uptodate, when this happens.
+
 =back
 
 =cut
@@ -521,6 +526,10 @@ sub stats   {
     return $_[0]->{TT}{po_in}->stats_get();
 }
 
+sub is_po_uptodate($) {
+    return $_[0]->{TT}{po_in}->equals_msgid($_[0]->{TT}{po_out});
+}
+
 =head2 Manipulating addenda
 
 =over 4
@@ -1102,7 +1111,7 @@ add an argument to translate to indicate the target language
 =item -
 
 make a pushline_all function, which would make pushline of its content for
-all language, using a map-like syntax:
+all languages, using a map-like syntax:
 
     $self->pushline_all({ "Description[".$langcode."]=".
                           $self->translate($line,$ref,$langcode)
